//+------------------------------------------------------------------+
//|                                              CS_Volty_EA_New.mq4 |
//|                        Copyright 2022, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2022, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict

//+------------------------------------------------------------------+
//--- returns account currency
#define _Deposit          AccountInfoString(ACCOUNT_CURRENCY)
//--- returns account balance
#define _Balance          AccountInfoDouble(ACCOUNT_BALANCE)
//--- returns account margin
#define _Margin           AccountInfoDouble(ACCOUNT_MARGIN)
//--- returns account free margin
#define _FreeMargin       AccountInfoDouble(ACCOUNT_MARGIN_FREE)
//--- returns account leverage
#define _Leverage         (int)AccountInfoInteger(ACCOUNT_LEVERAGE)
//--- returns symbol contract size
#define _ContractSize     SymbolInfoDouble(_Symbol, SYMBOL_TRADE_CONTRACT_SIZE)
//--- returns symbol lot step
#define _LotStep          SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_STEP)
//--- returns symbol lot min
#define _LotMin           SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MIN)
//--- returns symbol lot max
#define _LotMax           SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MAX)
//--- returns symbol tick value
#define _TickValue        SymbolInfoDouble(_Symbol, SYMBOL_TRADE_TICK_VALUE)
//--- returns symbol contract size
#define _TickSize         SymbolInfoDouble(_Symbol, SYMBOL_TRADE_TICK_SIZE)
//--- returns symbol pipValue
#define _PointValue       _TickValue*_Point / _TickSize
//--- returns symbol pip
#define _Pip              (double)((_Digits==3 || _Digits==5) ? _Point*10 : _Point)
//--- returns symbol pipValue
#define _PipValue         _TickValue*_Pip / _TickSize
//--- returns symbol pipValue
#define _BrokerSpread     (SymbolInfoDouble(_Symbol, SYMBOL_ASK)-SymbolInfoDouble(_Symbol, SYMBOL_BID))

//+------------------------------------------------------------------+
#define  EA_VERSION    "Rev.1.0 [2022.1.27]"
#define  EA_NAME       "RectBreakOut_EA"

#include <WinUser32.mqh>
#include <stdlib.mqh>
#import "stdlib.ex4"
string ErrorDescription(int error_code);
#import

enum ENUM_DIRECTION {NoOrder, Long, Short, Both};
enum ENUM_MONEY_MANAGEMENT {FixedNone, FixedLot, FixedPercent, FixedRisk, FixedMargin, SizeOptimized};

//--- input parameters
input string  _money   = "Money Management";
input ENUM_MONEY_MANAGEMENT   InpMMType   = FixedNone;      // Select MM Type
input double                  InpLots           = 0.1;            // Lot
input double                  InpLotPercent     = 1.5;            // Lot %, if 0 then use Lot
input double                  InpDecreaseFactor = 1.5;            // Decrease Factor

input string  _trade   = "Trade";
ENUM_DIRECTION          InpDirection      = NoOrder;        // Initial Trades, trigger
input int                     InpMaximumOrder   = 10;            // Maximum Order
input double                  InpTakeProfit     = 129.0;           // Take Profit
input double                  InpDistanceLimit  = 50;              // AddPips to pending level
input double                  InpAddPips        = 3;              // AddPips to pending level
input int                     InpSlippage       = 10;            // Slippage
input double                  InpMaxSpread      = 6.0;            // Max Spread we allowed
input bool                    InpOrdModOnEvent  = true;       // Modify Order On Rectangle Drag
input int                     InpMagic          = 234;            // magic number

input string  _marti   = "Martingale";
input bool                    InpUseMartingale  = true;           // martingale
input double                  InpMartiFactor    = 2.0;            // Martingale factor

input string  _graphics = "Graphics";
input bool                    HideRectangle     = false;          // Hide Rectangle
input bool                    ShowTrendLine     = true;          // Show Price Label
input bool                    ShowPriceLabel    = true;          // Show TrendLine
//---
/*
*/
//define

// V A R I A B L E S
//ulong m_slippage=10;                // slippage
double   ExtDistLimit=0.0;
double   ExtAddPips=0.0;
double   ExtTakeProfit=0.0;
double   ExtFirstTP=0.0;
double   ExtMaxSpread=0.0;
double   dAsk, dBid;
bool     m_init_error   = false;    // error on InInit
double   LongPrice, ShortPrice,
         LongStop, LongTake,
         ShortStop, ShortTake;
double   positionLot, orderLot, positionRiskInPoint;
bool     IsCurrPosLoss=false;
//int      totalPositions;
//int      totalOrders;
int      count_buy, count_sell, count_buystop, count_sellstop, count_buylimit, count_selllimit ;
int      count_positions, count_orders, count_all_orders ;
double   buy_op, sell_op, buystop_op, sellstop_op, buylimit_op, selllimit_op ;
double   buy_sl, sell_sl, buystop_sl, sellstop_sl, buylimit_sl, selllimit_sl;
double   buy_tp, sell_tp, buystop_tp, sellstop_tp, buylimit_tp, selllimit_tp;
double   buy_lot, sell_lot, buystop_lot, sellstop_lot, buylimit_lot, selllimit_lot;
double   buy_profit, sell_profit;
int      buy_ticket=-1, buystop_ticket=-1, buylimit_ticket=-1;
int      sell_ticket=-1, sellstop_ticket=-1, selllimit_ticket=-1;
double   NextLot;
double   balans, InitialLot;
int      LevelOP;
datetime t_start_cycle;
double   BSprice, BSstop, BStake, SSprice, SSstop, SStake;
double   BLprice, BLstop, BLtake, SLprice, SLstop, SLtake;

//indicators
bool     m_close_all=false;
double   BuyLevel=0.0;
double   SellLevel=0.0;
double   BuySL=0.0;
double   SellSL=0.0;
int      copied;
string   csvfname;
double   freeze_level;
double   stop_level;
double   ExtPoint;
//string msg;

//Graphics
string   prefix=_Symbol+"_";
string   sMiddleRectangle       = prefix+"MiddleRectangle";
string   sMiddleRectTLPrice     = prefix+"MiddleRectTLPrice";
string   sMiddleRectTLSL        = prefix+"MiddleRectTLSL";
string   sMiddleRectLabelPrice  = prefix+"MiddleRectLabelPrice";
string   sMiddleRectLabelSL     = prefix+"MiddleRectLabelSL";

string   sLongRectangle       = prefix+"LongRectangle";
string   sLongRectTLPrice     = prefix+"LongRectTLPrice";
string   sLongRectTLSL        = prefix+"LongRectTLSL";
string   sLongRectLabelPrice  = prefix+"LongRectLabelPrice";
string   sLongRectLabelSL     = prefix+"LongRectLabelSL";

string   sShortRectangle      = prefix+"ShortRectangle";
string   sShortRectTLPrice    = prefix+"ShortRectTLPrice";
string   sShortRectTLSL       = prefix+"ShortRectTLSL";
string   sShortRectLabelPrice = prefix+"ShortRectLabelPrice";
string   sShortRectLabelSL    = prefix+"ShortRectLabelSL";

string   sButt_BuyStop        = prefix+"Butt_BuyStop";
string   sButt_SellStop       = prefix+"Butt_SellStop";
string   sButt_BuyLimit       = prefix+"Butt_BuyLimit";
string   sButt_SellLimit      = prefix+"Butt_SellLimit";


string   sButt_ShowHide_lRect = prefix+"Butt_Show_lRect";
string   sButt_ShowHide_mRect = prefix+"Butt_Show_mRect";
string   sButt_ShowHide_sRect = prefix+"Butt_Show_sRect";
string   sButt_Exit           = prefix+"exit";

color    cMiddleRectColor   = clrIndigo;
color    cLongRectColor = (!HideRectangle) ? clrNavy : clrBlack;
color    cShortRectColor = (!HideRectangle) ? clrFireBrick : clrBlack;
color    BuyColor = clrDodgerBlue;
color    SellColor = clrDeepPink;
double   LongRectHigh, LongRectLow,
         MiddleRectHigh, MiddleRectLow,
         ShortRectHigh, ShortRectLow;
datetime LeftTimeLongRect, RightTimeLongRect,
         LeftTimeShortRect, RightTimeShortRect,
         LeftTimeMiddleRect, RightTimeMiddleRect,
         tempLeftTimeRect, tempRightTimeRect;
datetime t1, t0;
double   pLongHigh, pLongLow, pShortHigh, pShortLow;

//miscelanous
double temp=0.0;
double test=0.0;
string message;
datetime tempHighestBarTime, tempLowestBarTime,
         recentHighestBarTime, recentLowestBarTime;
int      firstSize;
int      newSize;
datetime prev_bars,
         last_signal;
int      bar_index;
datetime time_0;
datetime currentTime;
datetime leftTime;
bool opThisBar = false;
string msg;

int    PojokTxt = 0;
string FontTxt = "Tahoma";
int    SizeTxt = 8;
color  WarnaTxt = clrWhite;
int    MartiLevel;

double currUpperPrice, currLowerPrice;
string text[20];
bool stateHide=false;

int X[]= {
   62, //0 button buystop
   120, //1 button sellstop
   62, //2 button buylimit
   120, //3 button selllimit
   88, //4 show/hide long rectangle
   88, //5 show/hide middle rectangle
   88, //6 show/hide short rectangle
   20  //7 exit
};

int Y[]= {
   20, //0 button buystop
   20, //1 button sellstop
   38, //2 button buylimit
   38, //3 button selllimit
   98, //4 show/hide long rectangle
   78, //5 show/hide middle rectangle
   58, //6 show/hide short rectangle
   20  //7 exit
};

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
//---
   RefreshRates();

//--- tuning for 3 or 5 digits
//InitialLot     = (InpLotPercent>0) ? ((InpLotPercent/100)*_Balance)/(double)(_ContractSize*_Leverage) : InpLots;
   ExtDistLimit   = InpDistanceLimit * _Pip;
   ExtAddPips     = InpAddPips      * _Pip;
   ExtTakeProfit  = InpTakeProfit   * _Pip;
   ExtMaxSpread   = InpMaxSpread    * _Pip;

   OnInitTestVisual();
   onInitObject();
//---
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
//---
   if (  reason==REASON_PROGRAM || reason==REASON_REMOVE ||
         reason==REASON_CHARTCLOSE || reason==REASON_TEMPLATE ||
         reason==REASON_INITFAILED) {

      ObjectsDeleteAll(0, prefix);
   }
}
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
{
//---
   OrdersInformation();
   UpdateStat();

   Comment("test : ", (string)ObjectGetInteger(0, sShortRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS));
}
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,         // Event identifier
                  const long& lparam,   // Event parameter of long type
                  const double& dparam, // Event parameter of double type
                  const string& sparam  // Event parameter of string type
                 )
{
//--- the left mouse button has been pressed on the chart
   if(id==CHARTEVENT_OBJECT_DRAG) {
      if(sparam == sLongRectangle) {
         lRectdynamicLabelTL();
         if(!CheckSpreadForTrade()) return;
//--- handle buylimit
         if(count_selllimit>0) {
            if(OrderSelect(selllimit_ticket, SELECT_BY_TICKET, MODE_TRADES)) {
               if(selllimit_op!=SLprice || selllimit_sl!=SLstop) {
                  OrderModify(selllimit_ticket, SLprice, SLstop, SLtake, 0, clrNONE);
               }
            }
         }
      }
      if(sparam == sMiddleRectangle) {
         mRectdynamicLabelTL();
         if(!CheckSpreadForTrade()) return;
//--- handle buystop
         if(count_buystop>0) {
            if(OrderSelect(buystop_ticket, SELECT_BY_TICKET, MODE_TRADES)) {
               if(buystop_op!=BSprice || buystop_sl!=BSstop) {
                  OrderModify(buystop_ticket, BSprice, BSstop, BStake, 0, clrNONE);
               }
            }
         }
//--- handle sellstop
         if(count_sellstop>0) {
            if(OrderSelect(sellstop_ticket, SELECT_BY_TICKET, MODE_TRADES)) {
               if(sellstop_op!=SSprice || sellstop_sl!=SSstop) {
                  OrderModify(sellstop_ticket, SSprice, SSstop, SStake, 0, clrNONE);
               }
            }
         }
      }

      if(sparam == sShortRectangle) {
         sRectdynamicLabelTL();
         if(!CheckSpreadForTrade()) return;
//--- handle buylimit
         if(count_buylimit>0) {
            if(OrderSelect(buylimit_ticket, SELECT_BY_TICKET, MODE_TRADES)) {
               if(buylimit_op!=BLprice || buylimit_sl!=BLstop) {
                  OrderModify(buylimit_ticket, BLprice, BLstop, BLtake, 0, clrNONE);
               }
            }
         }

      }
   }
//////////////////////////// EVENT CLICK /////////////////////////////
   if(id==CHARTEVENT_OBJECT_CLICK) {
      //////////////////////////// EVENT CLICK HIDE /////////////////////////////
      if(sparam==sButt_ShowHide_lRect) {
         if(ObjectGetInteger(0,sparam,OBJPROP_STATE)) {
            ObjectSetInteger(0, sLongRectangle, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sLongRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sLongRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sLongRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sLongRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetString(0,sButt_ShowHide_lRect,OBJPROP_TEXT,"Show_lRect");
            stateHide=true;
         }
         else {
            stateHide=false;
            ObjectSetString(0,sButt_ShowHide_lRect,OBJPROP_TEXT,"Hide_lRect");
            ObjectSetInteger(0, sLongRectangle, OBJPROP_SELECTED, true);
            ObjectSetInteger(0, sLongRectangle, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sLongRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sLongRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sLongRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sLongRectTLSL, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ChartRedraw();
         }
      }
      if(sparam==sButt_ShowHide_mRect) {
         if(ObjectGetInteger(0,sparam,OBJPROP_STATE)) {
            ObjectSetInteger(0, sMiddleRectangle, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sMiddleRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sMiddleRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sMiddleRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sMiddleRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetString(0,sButt_ShowHide_mRect,OBJPROP_TEXT,"Show_mRect");
            stateHide=true;
         }
         else {
            stateHide=false;
            ObjectSetString(0,sButt_ShowHide_mRect,OBJPROP_TEXT,"Hide_mRect");
            ObjectSetInteger(0, sMiddleRectangle, OBJPROP_SELECTED, true);
            ObjectSetInteger(0, sMiddleRectangle, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sMiddleRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sMiddleRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sMiddleRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sMiddleRectTLSL, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ChartRedraw();
         }
      }
      if(sparam==sButt_ShowHide_sRect) {
         if(ObjectGetInteger(0,sparam,OBJPROP_STATE)) {
            ObjectSetInteger(0, sShortRectangle, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sShortRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sShortRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sShortRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetInteger(0, sShortRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
            ObjectSetString(0,sButt_ShowHide_mRect,OBJPROP_TEXT,"Show_sRect");
            stateHide=true;
         }
         else {
            stateHide=false;
            ObjectSetString(0,sButt_ShowHide_mRect,OBJPROP_TEXT,"Hide_sRect");
            ObjectSetInteger(0, sShortRectangle, OBJPROP_SELECTED, true);
            ObjectSetInteger(0, sShortRectangle, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sShortRectangle, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sShortRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sShortRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sShortRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ObjectSetInteger(0, sShortRectTLSL, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
            ChartRedraw();
         }
      }
      //////////////////////////// EVENT CLICK HIDE /////////////////////////////
      if(sparam == sButt_BuyStop) {
         if(ObjectGetInteger(0,sparam,OBJPROP_STATE)) {
         }
         
         if(count_buystop<InpMaximumOrder) {
            if(dAsk<BSprice) {
               OpenOrders(ORDER_TYPE_BUY_STOP, CalculateVolume(), BSprice, BSstop, BStake, clrAqua);
            }
         }
      }
      if(sparam == sButt_BuyLimit) {
         if(count_buylimit<InpMaximumOrder) {
            if(dAsk>BLprice) {
               OpenOrders(ORDER_TYPE_BUY_LIMIT, CalculateVolume(), BLprice, BLstop, BLtake, clrAqua);
            }
         }
      }
      if(sparam == sButt_SellStop) {
         if(count_sellstop<InpMaximumOrder) {
            if(dBid>SSprice) {
               OpenOrders(ORDER_TYPE_SELL_STOP, CalculateVolume(), SSprice, SSstop, SStake, clrMagenta);
            }
         }
      }
      if(sparam == sButt_SellLimit) {
         if(count_selllimit<InpMaximumOrder) {
            if(dBid<SLprice) {
               OpenOrders(ORDER_TYPE_SELL_LIMIT, CalculateVolume(), SLprice, SLstop, SLtake, clrMagenta);
            }
         }
      }
   }
}

//+------------------------------------------------------------------+
void OpenOrders(int op, double lot, double price, double sl, double tp, color col)
{
   int ticket = 0;
   int counter = 0, maxtry = 10;
   string EA_COMMENT = EA_NAME;
   lot = NormalizeLot(lot);
   price = NormalizeDouble(price, Digits);
   tp = NormalizeDouble(tp, Digits);
   sl = NormalizeDouble(sl, Digits);

//if (AccountFreeMarginCheck(Symbol(), op-4, lotnya)<=0 || GetLastError()==134) {
// Display_Err(134);
// return;  }

   for (counter = 0; counter < maxtry; counter++) {
      ticket = OrderSend(Symbol(), op, lot, price, InpSlippage, sl, tp, EA_COMMENT, InpMagic, 0, col);
      if (ticket > 0/* NO_ERROR */) {
         opThisBar = false;
         return;
      }
   }
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OrdersInformation()
{
   for (int c=0; c < OrdersTotal(); c++ ) {
      if (OrderSelect(c, SELECT_BY_POS, MODE_TRADES)) {
         if (OrderSymbol() != Symbol())continue;
         if (OrderMagicNumber() != InpMagic)continue;
         switch (OrderType()) {
         case OP_BUY :
            count_buy++;
            buy_ticket=OrderTicket();
            buy_lot=OrderLots();
            buy_sl = OrderStopLoss();
            buy_op = OrderOpenPrice();
            buy_tp = OrderTakeProfit();
            positionLot =  OrderLots();
            IsCurrPosLoss = buy_op>buy_sl ? true : false;
            positionRiskInPoint = (buy_op-buy_sl)/_Point;
            break;
         case OP_SELL :
            count_sell++;
            sell_ticket=OrderTicket();
            sell_lot=OrderLots();
            sell_op = OrderOpenPrice();
            sell_sl = OrderStopLoss();
            sell_tp = OrderTakeProfit();
            positionLot =  OrderLots();
            IsCurrPosLoss = sell_sl>sell_op ? true : false;
            positionRiskInPoint = (sell_sl-sell_op)/_Point;
            break;
         case OP_BUYSTOP :
            count_buystop++;
            buystop_ticket=OrderTicket();
            buystop_lot=OrderLots();
            buystop_tp = OrderTakeProfit();
            buystop_sl = OrderStopLoss();
            buystop_op = OrderOpenPrice();
            break;
         case OP_SELLSTOP:
            count_sellstop++;
            sellstop_ticket=OrderTicket();
            sellstop_lot=OrderLots();
            sellstop_op = OrderOpenPrice();
            sellstop_sl = OrderStopLoss();
            sellstop_tp = OrderTakeProfit();
            break;
         case OP_BUYLIMIT :
            count_buylimit++;
            buylimit_ticket=OrderTicket();
            buylimit_lot=OrderLots();
            buylimit_sl = OrderStopLoss();
            buylimit_op = OrderOpenPrice();
            buylimit_tp = OrderTakeProfit();
            break;
         case OP_SELLLIMIT:
            count_selllimit++;
            selllimit_ticket=OrderTicket();
            selllimit_lot=OrderLots();
            selllimit_sl = OrderStopLoss();
            selllimit_op = OrderOpenPrice();
            selllimit_tp = OrderTakeProfit();
            break;
         }
         count_all_orders++;
      }
   }
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsVolumeCorrect()
{
   if((nextMartiVolume()>orderLot*0.9) || (nextMartiVolume()<orderLot*1.1))
      return (true);
   return(false);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double nextMartiVolume()
{
   double nextVolume = 0, price=0, stop=0, tempLot=0, loss=0,
          lotRecovery=0, riskFactor=0, currPositionLoss=0,
          riskInMoney=0, totalRiskInMoney=0, tempRiskInMoney=0;

   riskFactor  = InpLotPercent/100;
   riskInMoney = riskFactor*_Balance;
//tickRange   = (LongPrice - ShortPrice)/_Point;

//_Balance*riskFactor = _PointValue*positionRiskInPoint*positionLot;
   currPositionLoss = _PointValue*positionRiskInPoint*positionLot;
   tempRiskInMoney  = (FLoatingLossCycle()<0 || IsCurrPosLoss) ?
                      MathAbs(FLoatingLossCycle()+ currPositionLoss) : 0;

   /*
   if(OrderCalcProfit(ORDER_TYPE_BUY,_Symbol, 1.0, price, stop, loss) && loss < 0) {
      totalRiskInMoney = riskInMoney + tempRiskInMoney;
      tempLot          = totalRiskInMoney/MathAbs(loss);
   }
   */

   nextVolume  =  (InpMMType==FixedRisk) ? MathFloor(tempLot/_LotStep)*_LotStep :
                  (InpMMType==FixedPercent) ? (riskInMoney / (_ContractSize/_Leverage)) :
                  (InpMMType==FixedMargin) ? (riskInMoney/_Margin) :
                  (InpMMType==SizeOptimized) ? Optimize(riskFactor/_TickValue) : InpLots ;

   return NormalizeLot(nextVolume);
}
//+------------------------------------------------------------------+
//| Lots or risk in percent for a deal from last balance             |
//+------------------------------------------------------------------+
double CalculateVolume(/*, double take*/)
{
   double price=0, stop=0, riskInMoney=0, loss=0;
   double tempLot=0, volume = 0.0, tickRange = 0.0, riskFactor=0;

   riskFactor  = InpLotPercent/100;
   riskInMoney = riskFactor*_Balance;
   tickRange   = (LongPrice - ShortPrice)/_Point;
//double lots = (Risk * freeMargin) / (PipValue * Pips);
   /*
   if(OrderCalcProfit(ORDER_TYPE_BUY,_Symbol, 1.0, currentUpperPrice(), currentLowerPrice(), loss) && loss < 0) {
      tempLot = riskInMoney/MathAbs(loss);
   }
   */
   volume = (InpMMType==FixedRisk) ? riskInMoney/(_PointValue*tickRange) :
            (InpMMType==FixedPercent) ? (riskInMoney / (_ContractSize/_Leverage)) :
            (InpMMType==FixedMargin) ? (riskInMoney/_Margin) :
            (InpMMType==SizeOptimized) ? Optimize(riskFactor/_TickValue) : InpLots ;

   return NormalizeLot(volume);
}
//+------------------------------------------------------------------+
//| Optimizing lot size for open.                                    |
//+------------------------------------------------------------------+
double Optimize(double lot)
{
   double losses=0;
//--- calculate number of losses orders without a break
   if(InpDecreaseFactor>0) {
      //--- select history for access
      //---
      for (int c = OrdersHistoryTotal() - 1; c >=0; c--) {
         if (!OrderSelect(c, SELECT_BY_POS, MODE_HISTORY)) continue;
         if (OrderSymbol() != Symbol()) continue;
         if (OrderMagicNumber() != InpMagic) continue;
         //if (StringFind(OrderComment(), "tp") > 0) break;
         if (OrderLots() == 0) continue;
         if (OrderType() > OP_SELL) continue;
         if (OrderProfit() >=0) continue;
         losses += OrderProfit();
      }
      //---
      if(losses>1)
         lot=NormalizeDouble(lot-lot*losses/InpDecreaseFactor,2);
   }
//--- normalize and check limits
   lot=_LotStep*NormalizeDouble(lot/_LotStep,0);
//---
   if(lot<_LotMin)
      lot=_LotMin;
//---
   if(lot>_LotMax)
      lot=_LotMax;
//---
   return(lot);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double FLoatingLossCycle()
{
   double losses=0;
   for (int c = OrdersHistoryTotal() - 1; c >=0; c--) {
      if (!OrderSelect(c, SELECT_BY_POS, MODE_HISTORY)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != InpMagic) continue;
      //if (StringFind(OrderComment(), "tp") > 0) break;
      if (OrderLots() == 0) continue;
      if (OrderType() > OP_SELL) continue;
      if (OrderProfit() >=0) continue;
      losses += OrderProfit();
   }
   return losses;
}
//+------------------------------------------------------------------+
//| Create Rectangle OnInit                                          |
//+------------------------------------------------------------------+
void onInitObject()
{
   t0=iTime(_Symbol, PERIOD_CURRENT, 0);
   t1=iTime(_Symbol, PERIOD_CURRENT, 21);

   pLongHigh = Bid - ExtDistLimit;
   pLongLow = pLongHigh - 40*_Pip;
   pShortLow = Bid + ExtDistLimit;
   pShortHigh = pShortLow + 40*_Pip;

   double pMidLow  = Bid - 25*_Pip;
   double pMidHigh = Bid + 25*_Pip;

   if(!createRectangle(sMiddleRectangle, cMiddleRectColor, t1, pMidLow, t0, pMidHigh)) return;
   if(!createRectangle(sLongRectangle, cLongRectColor, t1, pLongLow, t0, pLongHigh))  return;
   if(!createRectangle(sShortRectangle, cShortRectColor, t1, pShortHigh, t0, pShortLow)) return;;
//-----------------------------------------------------------------
   double dpi=(double)TerminalInfoInteger(TERMINAL_SCREEN_DPI)/96;


//Button_hide Shor rectangle
   CreateButton(sButt_ShowHide_lRect,OBJ_BUTTON,"Show_lRect",
                X[4]*dpi,   Y[4]*dpi,
                (int)(80*dpi),       (int)(15*dpi),
                cLongRectColor, clrWhite,
                clrPurple,9,CORNER_RIGHT_LOWER,false,false," ",false);

//Button_hide Shor rectangle
   CreateButton(sButt_ShowHide_mRect,OBJ_BUTTON,"Show_mRect",
                X[5]*dpi,   Y[5]*dpi,
                (int)(80*dpi),       (int)(15*dpi),
                cMiddleRectColor, clrWhite,
                clrPurple,9,CORNER_RIGHT_LOWER,false,false," ",false);

//Button_hide Shor rectangle
   CreateButton(sButt_ShowHide_sRect,OBJ_BUTTON,"Show_sRect",
                X[6]*dpi,   Y[6]*dpi,
                (int)(80*dpi),       (int)(15*dpi),
                cShortRectColor, clrWhite,
                clrPurple,9,CORNER_RIGHT_LOWER,false,false," ",false);

//Button_buystop
   CreateButton(sButt_BuyStop,OBJ_BUTTON,"BuyStop",
                X[0]*dpi,         Y[0]*dpi,
                (int)(55*dpi), (int)(15*dpi),
                clrDodgerBlue,clrWhite,
                clrDodgerBlue,9,CORNER_RIGHT_LOWER);
//Button_buystop
   CreateButton(sButt_SellStop,OBJ_BUTTON,"SellStop",
                X[1]*dpi,         Y[1]*dpi,
                (int)(55*dpi), (int)(15*dpi),
                clrMagenta,clrWhite,
                clrMagenta,9,CORNER_RIGHT_LOWER);
//Button_buystop
   CreateButton(sButt_BuyLimit,OBJ_BUTTON,"BuyLimit",
                X[2]*dpi,         Y[2]*dpi,
                (int)(55*dpi), (int)(15*dpi),
                clrDodgerBlue,clrWhite,
                clrDodgerBlue,9,CORNER_RIGHT_LOWER);
//Button_buystop
   CreateButton(sButt_SellLimit,OBJ_BUTTON,"SellLimit",
                X[3]*dpi,         Y[3]*dpi,
                (int)(55*dpi), (int)(15*dpi),
                clrMagenta,clrWhite,
                clrMagenta,9,CORNER_RIGHT_LOWER);

//ButtonCreate(sButt_Long, 124, 20, clrWhite, clrDodgerBlue);
//ButtonCreate(sButt_Short, 62, 20, clrWhite, clrMagenta);
   /*
   */
}
//+------------------------------------------------------------------+
//| Handle Rectangle OnChartEvent                                    |
//+------------------------------------------------------------------+
void GetRecValue()
{

   double   pLongRect1, pLongRect2,
            pMiddleRect1, pMiddleRect2,
            pShortRect1, pShortRect2;
   datetime tLongRect1, tLongRect2,
            tMiddleRect1, tMiddleRect2,
            tShortRect1, tShortRect2;

   pLongRect1     = ObjectGetDouble(0, sLongRectangle, OBJPROP_PRICE, 0);
   pLongRect2     = ObjectGetDouble(0, sLongRectangle, OBJPROP_PRICE, 1);
   tLongRect1     = (datetime) ObjectGetInteger(0, sLongRectangle, OBJPROP_TIME, 0);
   tLongRect2     = (datetime) ObjectGetInteger(0, sLongRectangle, OBJPROP_TIME, 1);
   LongRectHigh      = MathMax(pLongRect1, pLongRect2);
   LongRectLow       = MathMin(pLongRect1, pLongRect2);
   LeftTimeLongRect    = MathMin(tLongRect1, tLongRect2);
   RightTimeLongRect   = MathMax(tLongRect1, tLongRect2);

   pMiddleRect1   = ObjectGetDouble(0, sMiddleRectangle, OBJPROP_PRICE, 0);
   pMiddleRect2   = ObjectGetDouble(0, sMiddleRectangle, OBJPROP_PRICE, 1);
   tMiddleRect1   = (datetime) ObjectGetInteger(0, sMiddleRectangle, OBJPROP_TIME, 0);
   tMiddleRect2   = (datetime) ObjectGetInteger(0, sMiddleRectangle, OBJPROP_TIME, 1);
   MiddleRectHigh    = MathMax(pMiddleRect1, pMiddleRect2);
   MiddleRectLow     = MathMin(pMiddleRect1, pMiddleRect2);
   LeftTimeMiddleRect   = MathMin(tMiddleRect1, tMiddleRect2);
   RightTimeMiddleRect  = MathMax(tMiddleRect1, tMiddleRect2);

   pShortRect1    = ObjectGetDouble(0, sShortRectangle, OBJPROP_PRICE, 0);
   pShortRect2    = ObjectGetDouble(0, sShortRectangle, OBJPROP_PRICE, 1);
   tShortRect1    = (datetime) ObjectGetInteger(0, sShortRectangle, OBJPROP_TIME, 0);
   tShortRect2    = (datetime) ObjectGetInteger(0, sShortRectangle, OBJPROP_TIME, 1);
   ShortRectHigh     = MathMax(pShortRect1, pShortRect2);
   ShortRectLow      = MathMin(pShortRect1, pShortRect2);
   LeftTimeShortRect    = MathMin(tShortRect1, tShortRect2);
   RightTimeShortRect   = MathMax(tShortRect1, tShortRect2);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void getPriceVavue()
{
   GetRecValue();

   t0=iTime(_Symbol, PERIOD_CURRENT, 0);
   t1=iTime(_Symbol, PERIOD_CURRENT, 21);

   BSprice   = NormalizePrice(MiddleRectHigh + _BrokerSpread + ExtAddPips);
   BSstop    = NormalizePrice(MiddleRectLow - ExtAddPips);
   BStake    = NormalizePrice(BSprice + ExtTakeProfit);

   SSprice   = NormalizePrice(MiddleRectLow - ExtAddPips);
   SSstop    = NormalizePrice(MiddleRectHigh + _BrokerSpread + ExtAddPips);
   SStake    = NormalizePrice(SSprice - ExtTakeProfit);

   BLprice   = NormalizePrice(LongRectHigh + _BrokerSpread + ExtAddPips);
   BLstop    = NormalizePrice(LongRectLow - ExtAddPips);
   BLtake    = NormalizePrice(BLprice + ExtTakeProfit);

   SLprice   = NormalizePrice(ShortRectLow - ExtAddPips);
   SLstop    = NormalizePrice(ShortRectHigh + _BrokerSpread + ExtAddPips);
   SLtake    = NormalizePrice(SLprice - ExtTakeProfit);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void lRectdynamicLabelTL()
{
   getPriceVavue();
   if(ObjectFind(0, sLongRectangle)!=-1 ) {
      if(ShowPriceLabel) {
         priceLabel(sLongRectLabelPrice, SellColor, RightTimeLongRect, BLprice);
         priceLabel(sLongRectLabelSL, BuyColor, RightTimeLongRect, BLstop);
      }
      if(ShowTrendLine) {
         dynamicTrendline(sLongRectTLPrice, SellColor, LeftTimeLongRect, BLprice, RightTimeLongRect, BLprice); //okay
         dynamicTrendline(sLongRectTLSL, BuyColor, LeftTimeLongRect, BLstop, RightTimeLongRect, BLstop); //okay
      }
   }
   ObjectSetInteger(0, sShortRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sShortRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sShortRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sShortRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void mRectdynamicLabelTL()
{
   getPriceVavue();
   if(ObjectFind(0, sMiddleRectangle)!=-1 ) {
      if(ShowPriceLabel) {
         priceLabel(sMiddleRectLabelPrice, BuyColor, RightTimeMiddleRect, BSprice);
         priceLabel(sMiddleRectLabelSL, SellColor, RightTimeMiddleRect, SSprice);
      }
      if(ShowTrendLine) {
         dynamicTrendline(sMiddleRectTLPrice, BuyColor, LeftTimeMiddleRect, BSprice, RightTimeMiddleRect, BSprice); //okay
         dynamicTrendline(sMiddleRectTLSL, SellColor, LeftTimeMiddleRect, SSprice, RightTimeMiddleRect, SSprice); //okay
      }
   }
   ObjectSetInteger(0, sLongRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sLongRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sShortRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sShortRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sLongRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sLongRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sShortRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sShortRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void sRectdynamicLabelTL()
{
   getPriceVavue();
   if(ObjectFind(0, sShortRectangle)!=-1) {
      if(ShowPriceLabel) {
         priceLabel(sShortRectLabelPrice, SellColor, RightTimeShortRect, SLprice);
         priceLabel(sShortRectLabelSL, BuyColor, RightTimeShortRect, SLstop);
      }
      if(ShowTrendLine) {
         dynamicTrendline(sShortRectTLPrice, SellColor, LeftTimeShortRect, SLprice, RightTimeShortRect, SLprice); //okay
         dynamicTrendline(sShortRectTLSL, BuyColor, LeftTimeShortRect, SLstop, RightTimeShortRect, SLstop);  //okay
      }
   }
   ObjectSetInteger(0, sLongRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sLongRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectLabelPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectLabelSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sLongRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sLongRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectTLPrice, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
   ObjectSetInteger(0, sMiddleRectTLSL, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
}
//+------------------------------------------------------------------+
// Move rectangle                                                    |
//+------------------------------------------------------------------+
bool dynamicRectangle(string   &name,      // line name
                      color     &clr,
                      datetime  &time1,
                      double    &price1,
                      datetime  &time2,
                      double    &price2
                     )
{
//--- reset the error value
   ResetLastError();
//--- move a rectangle by the given coordinates
   if(ObjectFind(0, name)!=-1) {
      if(!ObjectMove(0, name, 0, time1, price1) || !ObjectMove(0, name, 1, time2, price2)) {
         Print(__FUNCTION__, ": failed to move the anchor point! Error code = ", GetLastError());
         return(false);
      }
   }
//--- create a rectangle by the given coordinates
   else  {
      if(!ObjectCreate(0, name, OBJ_RECTANGLE, 0, time1, price1, time2, price2)) {
         Print(__FUNCTION__, ": failed to create a rectangle! Error code = ", GetLastError());
         return(false);
      }
   }
//--- set line color
   ObjectSetInteger(0, name, OBJPROP_COLOR, clr);
//--- set line display style
   ObjectSetInteger(0, name, OBJPROP_STYLE, STYLE_DASH);
//--- set line width
   ObjectSetInteger(0, name, OBJPROP_WIDTH, 1);
//--- display in the foreground (false) or background (true)
//--- enable (true) or disable (false) the mode of filling the rectangle
   ObjectSetInteger(0, name, OBJPROP_FILL, true);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(0, name, OBJPROP_BACK, true);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(0, name, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
//ObjectSetInteger(0,name,OBJPROP_HIDDEN,true);
//ObjectSetInteger(_Symbol,name,OBJPROP_HIDDEN,hidden);
   return(true);
}
//+------------------------------------------------------------------+
// Move rectangle                                                    |
//+------------------------------------------------------------------+
bool createRectangle(string   &name,      // line name
                     color     &clr,
                     datetime  &time1,
                     double    &price1,
                     datetime  &time2,
                     double    &price2
                    )
{
//--- reset the error value
   ResetLastError();
//--- create a rectangle by the given coordinates
   if(ObjectFind(0, name)==-1) {
      //--- successful execution
      if(!ObjectCreate(0, name, OBJ_RECTANGLE, 0, time1, price1, time2, price2)) {
         Print(__FUNCTION__, ": failed to create a rectangle! Error code = ", GetLastError());
         return(false);
      }
   }
//--- set line color
   ObjectSetInteger(0, name, OBJPROP_COLOR, clr);
//--- set line display style
   ObjectSetInteger(0, name, OBJPROP_STYLE, STYLE_DASH);
   ObjectSetInteger(0, name, OBJPROP_SELECTABLE, true);
   ObjectSetInteger(0, name, OBJPROP_SELECTED, true);
//--- set line width
   ObjectSetInteger(0, name, OBJPROP_WIDTH, 1);
//--- display in the foreground (false) or background (true)
//--- enable (true) or disable (false) the mode of filling the rectangle
   ObjectSetInteger(0, name, OBJPROP_FILL, true);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(0, name, OBJPROP_BACK, true);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(0, name, OBJPROP_TIMEFRAMES, OBJ_NO_PERIODS);
//ObjectSetInteger(0,name,OBJPROP_HIDDEN,true);
//ObjectSetInteger(_Symbol,name,OBJPROP_HIDDEN,hidden);
   return(true);
}

//+------------------------------------------------------------------+
//| Create the right price label                                     |
//+------------------------------------------------------------------+
bool priceLabel(const string     &name, // price label name
                const color      &clr,         // priority for mouse click
                datetime         &time,            // anchor point time
                double           &price
               )
{
//--- set anchor point coordinates if they are not set
//--- reset the error value
   ResetLastError();
//--- create a price label
   if(ObjectFind(0, name) == -1) {
      if(!ObjectCreate(0, name, OBJ_ARROW_RIGHT_PRICE, 0, time, price)) {
         Print(__FUNCTION__, ": failed to create the right price label! Error code = ", GetLastError());
         return(false);
      }
   }
   else if(!ObjectMove(0, name, 0, time, price)) {
      Print(__FUNCTION__, ": failed to move the anchor point! Error code = ", GetLastError());
      return(false);
   }
//--- successful execution
//--- set the label color
   ObjectSetInteger(0, name, OBJPROP_COLOR, clr);
//--- set the border line style
   ObjectSetInteger(0, name, OBJPROP_STYLE, STYLE_SOLID);
//--- set the label size
   ObjectSetInteger(0, name, OBJPROP_WIDTH, 1);
//ObjectSetString(0,name,OBJPROP_FONT,font);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(0, name, OBJPROP_BACK, true);
//--- enable (true) or disable (false) the mode of moving the label by mouse
//--- when creating a graphical object using ObjectCreate function, the object cannot be
//--- highlighted and moved by default. Inside this method, selection parameter
//--- is true by default making it possible to highlight and move the object
//ObjectSetInteger(0,name,OBJPROP_SELECTABLE,true);
//ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,false);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(0, name, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
//ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
//--- set the priority for receiving the event of a mouse click in the chart
//ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution
   return(true);
}
//+------------------------------------------------------------------+
// TrendLine Handling                                                |
//+------------------------------------------------------------------+
bool dynamicTrendline(string    &name,   // line name
                      color     &clr,
                      datetime  &time1,
                      double    &price1,
                      datetime  &time2,
                      double    &price2
                     )
{
   if(ObjectFind(0, name) == -1) {
      if(!ObjectCreate(0, name, OBJ_TREND, 0, time1, price1, time2, price2)) {
         Print(__FUNCTION__, ": failed to create a trend line! Error code = ", GetLastError());
         return(false);
      }
   }
//--- successful execution
   else if(!ObjectMove(0, name, 0, time1, price1) || !ObjectMove(0, name, 1, time2, price2)) {
      Print(__FUNCTION__, ": failed to move trendline! Error code = ", GetLastError());
      return(false);
   }
//--- set line color
   ObjectSetInteger(0, name, OBJPROP_COLOR, clr);
//--- set line display style
   ObjectSetInteger(0, name, OBJPROP_STYLE, STYLE_DOT);
//--- set line width
   ObjectSetInteger(0, name, OBJPROP_WIDTH, 1);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(0, name, OBJPROP_BACK, true);
//--- enable (true) or disable (false) the mode of continuation of the line's display to the right
   ObjectSetInteger(0, name, OBJPROP_RAY_RIGHT, false);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(0, name, OBJPROP_TIMEFRAMES, OBJ_ALL_PERIODS);
//--- successful execution
   return(true);
}
//+------------------------------------------------------------------+
//| Return the flag of a prefixed object presence                    |
//+------------------------------------------------------------------+
bool IsPresentObjects(const string object_prefix)
{
   for(int i=ObjectsTotal(0, 0)-1; i>=0; i--)
      if(StringFind(ObjectName(0, i, 0), object_prefix)>WRONG_VALUE)
         return true;
   return false;
}
//+------------------------------------------------------------------+
void OnInitTestVisual()
{
   if (IsVisualMode() || (!IsTesting())) {
      long hwnd=ChartID();
      if(hwnd>0) { // If it succeeded, additionally customize
         //--- Set the indent of the right border of the chart
         ChartSetInteger(hwnd,CHART_SHIFT,true);
         //--- Display as candlesticks
         ChartSetInteger(hwnd,CHART_MODE,CHART_CANDLES);
         ChartSetInteger(hwnd,CHART_SHOW_PERIOD_SEP, false);
         ChartSetInteger(hwnd,CHART_SHOW_GRID, false);
         ChartSetInteger(hwnd,CHART_FOREGROUND, false);
         ChartSetInteger(hwnd,CHART_BRING_TO_TOP,true);
         ChartSetInteger(hwnd,CHART_SHOW_ASK_LINE,true);
         ChartSetInteger(hwnd,CHART_SHIFT,true);
         ChartSetInteger(hwnd,CHART_AUTOSCROLL,true);
         ChartSetDouble(hwnd,CHART_SHIFT_SIZE,20);
         //--- Set the tick volume display mode
         //ChartSetInteger(handle,CHART_SHOW_VOLUMES,CHART_VOLUME_TICK);
      }
   }
}
//+------------------------------------------------------------------+
bool isNewBar(datetime newbar_time)
{
//--- Initialization of variables
   int new_bars = 0;
   datetime lastbar_time=0;
//--- Just to be sure, check: is the time of (hypothetically) new bar (newbar_time) less than time of last bar (lastbar_time)?
   if(lastbar_time>newbar_time) {
      return(false);
   }
//--- if it's the first call
   if(lastbar_time==0) {
      lastbar_time=newbar_time; //--- set time of last bar and exit
      return(false);
   }
//--- Check for new bar:
   if(lastbar_time<newbar_time) {
      new_bars=1;               // Number of new bars
      lastbar_time=newbar_time; // remember time of last bar
      return(true);
   }
//--- if we've reached this line, then the bar is not new; return false
   return(false);
}
//+------------------------------------------------------------------+
double NormalizePrice(double p, string pair="")
{
// https://forum.mql4.com/43064#515262 zzuegg reports for non-currency DE30:
// MarketInfo(chart.symbol,MODE_TICKSIZE) returns 0.5
// MarketInfo(chart.symbol,MODE_DIGITS) return 1
// Point = 0.1
// Prices to open must be a multiple of ticksize
   ResetLastError();
   if (pair == "") pair = Symbol();
   double tickSize = SymbolInfoDouble(pair,SYMBOL_TRADE_TICK_SIZE);
   return( MathRound(p/tickSize) * tickSize );
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double NormalizeLot(double lots)
{
   ResetLastError();
   double lotStep = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_STEP);
   double minLot = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MIN);
   lots = MathRound(lots/lotStep) * lotStep;
   if (lots < minLot) lots = 0;    // or minLot
   return(lots);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsNewBar(const ENUM_TIMEFRAMES tf = PERIOD_CURRENT)
{
   datetime bar_time=0 ;
   if (bar_time < iTime(Symbol(), tf, 0)) {
      bar_time = iTime(Symbol(), tf, 0);
      return (true);
   }
   return (false);
}
//+------------------------------------------------------------------+
//|                        CheckSpreadForTrade                       |
//+------------------------------------------------------------------+
bool CheckSpreadForTrade()
{
//-- if Broker Spread > Max Spread we are allowed
   if(ExtMaxSpread < _BrokerSpread) {
      string msgTemp = "Broker spread "+(string)_BrokerSpread+" at "+(string)_Symbol+
                       " has larger than max spread = "+(string)ExtMaxSpread+" we allowed" + "\n";
      msgTemp += "We wait until broker spread reduced.";
   }
//--- checking successful
   return(true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CreateButton(string name,ENUM_OBJECT Type,
                  string text_t,
                  int XDistance,
                  int YDistance,
                  int Width,
                  int Height,
                  color BGColor_,
                  color InfoColor,
                  color boarderColor,
                  int fontsize,
                  int corner=0,
                  bool readonly=false,
                  bool Obj_Selectable=false,
                  string Tooltip="",
                  bool state_t=false)
{
   ObjectCreate(0,name,Type,0,0,0);
   ObjectSetInteger(0,name,OBJPROP_XDISTANCE,XDistance);
   ObjectSetInteger(0,name,OBJPROP_YDISTANCE,YDistance);
   ObjectSetInteger(0,name,OBJPROP_XSIZE,Width);
   ObjectSetInteger(0,name,OBJPROP_YSIZE,Height);
   ObjectSetString(0,name,OBJPROP_TEXT,text_t);
   ObjectSetString(0,name,OBJPROP_FONT,"Arial Bold");
   ObjectSetInteger(0,name,OBJPROP_FONTSIZE,fontsize);
   ObjectSetInteger(0,name,OBJPROP_CORNER,corner);
   ObjectSetInteger(0,name,OBJPROP_COLOR,InfoColor);
   ObjectSetInteger(0,name,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,name,OBJPROP_BORDER_COLOR,boarderColor);
   ObjectSetInteger(0,name,OBJPROP_BGCOLOR,BGColor_);
   ObjectSetInteger(0,name,OBJPROP_SELECTABLE,Obj_Selectable);
   ObjectSetString(0,name,OBJPROP_TOOLTIP,Tooltip);
   ObjectSetInteger(0,name,OBJPROP_STATE,state_t);
   ObjectSetInteger(0,name,OBJPROP_ZORDER,1);

}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
void UpdateStat()
{
   myComment(1, "Balance", DoubleToStr(AccountBalance(),2), clrWhite);
   myComment(2, "Equity", DoubleToStr(AccountEquity(),2), clrWhite);
   myComment(3, "AccountProfit", DoubleToStr(AccountProfit(), 2), clrWhite);

//Long
   if(count_buy>0) {
      myComment(5, "Buy Position", "-- LONG --", clrWhite);
      myComment(6, "Lot Buy", DoubleToStr(buy_lot,2), clrWhite);
      myComment(7, "Buy Profit", DoubleToStr(buy_profit,2), clrWhite);
   }
   else if(count_buystop>0) {
      myComment(5, "BuyStop Order", "-- LONG --", clrWhite);
      myComment(6, "Lot BuyStop", DoubleToStr(buystop_lot,2), clrWhite);
      myComment(7, "BuyStop Profit", "---", clrWhite);
   }
   else if(count_buylimit>0) {
      myComment(5, "BuyLimit Order", "-- LONG --", clrWhite);
      myComment(6, "Lot BuyLimit", DoubleToStr(buylimit_lot,2), clrWhite);
      myComment(7, "BuyLimit Profit", "---", clrWhite);
   }
   else  {
      myComment(5, "No Order", "-- LONG --", clrWhite);
      myComment(6, "Lot ", "---", clrWhite);
      myComment(7, "Profit", "---", clrWhite);
   }
//Short
   if(count_sell>0) {
      myComment(9, "Sell Position", "-- SHORT --", clrWhite);
      myComment(10, "Lot Sell", DoubleToStr(sell_lot,2), clrWhite);
      myComment(11, "Sell Profit", DoubleToStr(sell_profit,2), clrWhite);
   }
   else if(count_sellstop>0) {
      myComment(9, "SellStop Order", "-- SHORT --", clrWhite);
      myComment(10, "Lot SellStop", DoubleToStr(sellstop_lot,2), clrWhite);
      myComment(11, "SellStop Profit", "---", clrWhite);
   }
   else if(count_selllimit>0) {
      myComment(9, "BuyLimit Order", "-- SHORT --", clrWhite);
      myComment(10, "Lot BuyLimit", DoubleToStr(selllimit_lot,2), clrWhite);
      myComment(11, "BuyLimit Profit", "---", clrWhite);
   }
   else  {
      myComment(9, "No Order", "-- SHORT --", clrWhite);
      myComment(10, "Lot ", "---", clrWhite);
      myComment(11, "Profit", "---", clrWhite);
   }

   myComment(13, "Spread", DoubleToStr(_BrokerSpread/_Pip,2), clrWhite);
   string add="";
   if (AccountMargin() > 0) {
      myComment(14, "MarginLevel", DoubleToStr(100 * AccountEquity() / AccountMargin(), 2) + " %", clrWhite);
   }
   myComment(15, "Level", IntegerToString(MathMax(0, MartiLevel)), clrWhite);

   double d, wd;
   GetDD_Worst(d, wd);
//myComment(16, "Drawdown ", DoubleToStr(d,2), clrWhite);
//myComment(17, "Worst_DD ", DoubleToStr(wd,2), clrWhite);
//myComment(16, "Target", DS(TargetEquity, 2), clrWhite);
   myComment(16, "Lot", DoubleToString(CalculateVolume(), 2), clrWhite);

}
//+------------------------------------------------------------------+
void Display_Err(int err = 0, string info="")
{
   SizeTxt = 12;
   PojokTxt = 3;

   if (err == 0) {
      if (info != "") {
         WarnaTxt = clrRed;
         Write("infoB", 10, 10, "Info : " + info);
      }
      else {
         WarnaTxt = clrLimeGreen;
         Write("infoB", 10, 10, "No Error");
      }
   }
   else {
      Print("Error: ", ErrorDescription(err));
      WarnaTxt = clrRed;
      Write("infoB", 10, 8, "Error : " + ErrorDescription(err));
   }
}
//+------------------------------------------------------------------+
void myComment (int baris, string label1, string label2, color warna)
{
   string msg;
   CanTrade(msg);
   int x = 15,
       y = 35+baris*15;
   SizeTxt = 12;
   PojokTxt = 1;
   WarnaTxt = clrYellow;
   Write("infoA", 10, 25, EA_NAME + " " + EA_VERSION + " "  + msg);
   SizeTxt = 8;
   PojokTxt = 1;
   WarnaTxt = warna;
   Write("infoX"+(string)baris, x, y, label2);
   Write("infoY"+(string)baris, x+70, y,":");
   Write("infoZ"+(string)baris, x+80, y, label1);
}
//+------------------------------------------------------------------+
void Write(string obj, int x, int y, string txt)
{
   if (ObjectFind(obj) == -1) {
      ObjectCreate(obj, OBJ_LABEL, 0, 0, 0, 0);
      ObjectSet(obj, OBJPROP_XDISTANCE, x);
      ObjectSet(obj, OBJPROP_YDISTANCE, y);
      //ObjectSet("Pair"+i,OBJPROP_COLOR, DodgerBlue);
      ObjectSet(obj, OBJPROP_CORNER, PojokTxt);
   }
   ObjectSetText(obj, txt, SizeTxt, FontTxt, WarnaTxt);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CanTrade(string &msg)
{
   if (IsTesting()) {
      msg = "run on strategy tester";
      return (true);
   }
   if (IsTradeAllowed()) {
      if (IsConnected()) {
         if (!IsTradeContextBusy()) {
            if (!IsStopped()) {
               msg = "trading aktif";
            }
            else {
               msg = "lagi proses berhenti...";
            }
         }
         else {
            msg = "belom dapet giliran trading";
         }
      }
      else {
         msg = "ga konek ke server";
      }
   }
   else {
      msg = "ga dibolehin trading";
   }
   return (false);
}
//+------------------------------------------------------------------+
void GetDD_Worst(double &Drawdown, double &wdd)
{
   static double WorstDrawdown = 0;
   Drawdown=0.0;
   if (AccountEquity() < AccountBalance()) {
      Drawdown = 100 - AccountEquity() / AccountBalance() * 100;
   }
   if (Drawdown>WorstDrawdown) WorstDrawdown=Drawdown;
   wdd = WorstDrawdown;
}
//+------------------------------------------------------------------+
